@@ -14,7 +14,6 @@ from twisted.python.url import URL
 
 from twisted.web.http import urlparse
 
-
 from twisted.web.http_headers import Headers
 from twisted.web.iweb import IBodyProducer, IResponse
 
@@ -28,6 +27,7 @@ from twisted.web.client import (
 )
 
 from twisted.python.components import registerAdapter
+from json import dumps as json_dumps
 
 from treq._utils import default_reactor
 from treq.auth import add_auth
@@ -106,24 +106,45 @@ class HTTPClient(object):
         self._data_to_body_producer = data_to_body_producer
 
     def get(self, url, **kwargs):
+        """
+        See :func:`treq.get()`.
+        """
         return self.request('GET', url, **kwargs)
 
     def put(self, url, data=None, **kwargs):
+        """
+        See :func:`treq.put()`.
+        """
         return self.request('PUT', url, data=data, **kwargs)
 
     def patch(self, url, data=None, **kwargs):
+        """
+        See :func:`treq.patch()`.
+        """
         return self.request('PATCH', url, data=data, **kwargs)
 
     def post(self, url, data=None, **kwargs):
+        """
+        See :func:`treq.post()`.
+        """
         return self.request('POST', url, data=data, **kwargs)
 
     def head(self, url, **kwargs):
+        """
+        See :func:`treq.head()`.
+        """
         return self.request('HEAD', url, **kwargs)
 
     def delete(self, url, **kwargs):
+        """
+        See :func:`treq.delete()`.
+        """
         return self.request('DELETE', url, **kwargs)
 
     def request(self, method, url, **kwargs):
+        """
+        See :func:`treq.request()`.
+        """
         method = method.encode('ascii').upper()
 
         # Join parameters provided in the URL
@@ -142,23 +163,9 @@ class HTTPClient(object):
             if isinstance(headers, dict):
                 h = Headers({})
                 for k, v in headers.items():
-
-                    if isinstance(k, unicode):
-                        k = k.encode('ascii')
-
-                    if isinstance(v, bytes):
+                    if isinstance(v, (bytes, unicode)):
                         h.addRawHeader(k, v)
-                    elif isinstance(v, unicode):
-                        h.addRawHeader(k, v.encode('ascii'))
                     elif isinstance(v, list):
-                        cleanHeaders = []
-                        for item in v:
-                            if isinstance(item, unicode):
-                                cleanHeaders.append(item.encode('ascii'))
-                            else:
-                                cleanHeaders.append(item)
-                        h.setRawHeaders(k, cleanHeaders)
-                    else:
                         h.setRawHeaders(k, v)
 
                 headers = h
@@ -170,6 +177,10 @@ class HTTPClient(object):
         bodyProducer = None
         data = kwargs.get('data')
         files = kwargs.get('files')
+        # since json=None needs to be serialized as 'null', we need to
+        # explicitly check kwargs for this key
+        has_json = 'json' in kwargs
+
         if files:
             # If the files keyword is present we will issue a
             # multipart/form-data request as it suits better for cases
@@ -194,6 +205,13 @@ class HTTPClient(object):
                     b'content-type', [b'application/x-www-form-urlencoded'])
                 data = urlencode(data, doseq=True)
             bodyProducer = self._data_to_body_producer(data)
+        elif has_json:
+            # If data is sent as json, set Content-Type as 'application/json'
+            headers.setRawHeaders(
+                b'content-type', [b'application/json; charset=UTF-8'])
+            content = kwargs['json']
+            json = json_dumps(content, separators=(u',', u':')).encode('utf-8')
+            bodyProducer = self._data_to_body_producer(json)
 
         cookies = kwargs.get('cookies', {})
 
